import sys
import argparse
from http.client import HTTPSConnection
from bs4 import BeautifulSoup
from replbuilder import ReplCommand


def get_pointer_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("pointer", type=int, help="Pointers from previous command")
    return parser


def hn_get(path, user_cookie):
    if not path.startswith("/"):
        path = "/" + path
    headers = {"Cookie": user_cookie}
    conn = HTTPSConnection("news.ycombinator.com")
    conn.request("GET", path, headers=headers)
    resp = conn.getresponse()
    if str(resp.code).startswith("4"):
        print("Call REJECTED: {}".format(resp.read()))
        raise "Call is unsuccessful"
    result = resp.read()
    conn.close()
    return result


def get_vote_stamp(vote_type):
    # I VOTED!
    if vote_type == "up":
        return "\033[1;32m[UPVOTED]\033[0m"
    elif vote_type == "down":
        return "\033[1;31m[DOWNVOTED]\033[0m"
    elif vote_type == "un":
        return "\033[1m[UNVOTED]\033[0m"
    else:
        return ""


def vote(args, context, vote_type):
    if context.user_cookie is None:
        raise ValueError("Cannot upvote if not logged in")
    item_id = context.current_pointers[args.pointer]
    sub_link = "item?id={}".format(item_id)
    item_page = hn_get(sub_link, context.user_cookie)
    soup = BeautifulSoup(item_page, "html.parser")
    optional_link = soup.select("a#{}_{}".format(vote_type, item_id))
    if not optional_link:
        print("Link to {}vote does not exist".format(vote_type))
        return
    vote_link = optional_link[0]
    if "nosee" in vote_link["class"]:
        if item_id in context.stored_votes:
            if context.stored_votes[item_id] != vote_type:
                print("This item is already {}".format(get_vote_stamp(context.stored_votes[item_id])))
                return
        print("{}vote cannot be enacted, you may have already voted".format(vote_type))
        context.store_vote(item_id, vote_type)
        return
    vote_path = vote_link["href"]
    hn_get(vote_path, context.user_cookie)
    context.store_vote(item_id, vote_type)
    print("POINTER {} is {}".format(args.pointer, get_vote_stamp(vote_type)))
    sys.stdout.flush()


def upvote(args, context):
    vote(args, context, "up")


def downvote(args, context):
    vote(args, context, "down")


def unvote(args, context):
    vote(args, context, "un")


upvote_command = ReplCommand("upvote", get_pointer_parser(), upvote, "Upvote story or comment indicated by pointer from previous commands", use_context=True)
downvote_command = ReplCommand("downvote", get_pointer_parser(), downvote, "Downvote comment indicated by pointer from previous commands", use_context=True)
unvote_command = ReplCommand("unvote", get_pointer_parser(), unvote, "Unvote comment that you have previously up or downvoted", use_context=True)



import argparse
from .hnclient import HNClient
from replbuilder import ReplCommand


class HNContext:

    HN_BASE_URL = "https://news.ycombinator.com/"

    @staticmethod
    def get_link_parser():
        parser = argparse.ArgumentParser()
        parser.add_argument("pointer", nargs="?", type=int, help="Get link by pointer")
        return parser

    def __init__(self):
        self.client = HNClient()
        self.current_pointers = {}
        self.story_list = []
        self.stored_votes = {}
        self.loaded_items = {}
        self.call_count = 0
        self.link = None
        self.user_cookie = None

    def get_story_list(self, story_type="TOP"):
        self.call_count += 1
        self.story_list = self.client.get_stories(story_type=story_type)

    def store_item(self, item_id):
        if item_id in self.loaded_items:
            return
        self.call_count += 1
        item = self.client.get_item(item_id)
        if item is None:
            raise ValueError("Provided id does not exist")
        self.loaded_items[item_id] = item

    def drop_item(self, item_id):
        if item_id in self.loaded_items:
            self.loaded_items.pop(item_id)

    def get_user_info(self, author_name):
        return self.client.get_user(author_name)

    def store_link(self, link):
        self.link = link

    def store_vote(self, item_id, vote_type):
        if vote_type == "un":
            if item_id in self.stored_votes:
                self.stored_votes.pop(item_id)
            return
        self.stored_votes[item_id] = vote_type

    def get_link(self, args):
        if args.pointer:
            item_id = self.current_pointers[args.pointer]
            sub_link = "item?id={}".format(item_id)
            print("\033[4;35m{}\033[0m".format(HNContext.HN_BASE_URL + sub_link))
        elif self.link is None:
            print("No link stored yet")
        else:
            print("\033[4;35m{}\033[0m".format(HNContext.HN_BASE_URL + self.link))

    def store_pointer(self, index, item_id):
        self.current_pointers[index] = item_id

    def clear_pointers(self):
        self.current_pointers = {}

    def get_cache(self, args):
        print("LOADED ITEMS COUNT : {}".format(len(self.loaded_items)))
        print("TOTAL HN API CALLS : {}".format(self.call_count))
        print("POINTERS COUNT     : {}".format(len(self.current_pointers)))
        print("STORED VOTES       : {}".format(len(self.stored_votes)))

    def clear_cache(self, args):
        self.loaded_items = {}
        self.current_pointers = {}
        # stored votes remain true, no need to update
        # call count will continue to increment

    def store_user_cookie(self, user_cookie):
        self.user_cookie = user_cookie

    def get_context_commands(self):
        return [
            ReplCommand("get_link", HNContext.get_link_parser(), self.get_link, "Get the browser link from pointers or from last command"),
            ReplCommand("get_cache", argparse.ArgumentParser(), self.get_cache, "See stored item count and call count"),
            ReplCommand("clear_cache", argparse.ArgumentParser(), self.clear_cache, "Remove all cache"),
        ]



from replbuilder import ReplRunner
from hnterminal.hncontext import HNContext
from hnterminal.commands import READ_COMMANDS, WRITE_COMMANDS


def build_cli():
    hn_context = HNContext()
    cli_runner = ReplRunner("hnterminal", hn_context)
    cli_runner.add_commands(READ_COMMANDS, namespace="Read")
    cli_runner.add_commands(WRITE_COMMANDS, namespace="Write")
    cli_runner.add_commands(hn_context.get_context_commands(), namespace="Cache")
    return cli_runner


def main():
    cli_runner = build_cli()
    cli_runner.run()


if __name__ == "__main__":
    main()



import os
import re
import sys
import time
import html
import argparse
import tempfile
from subprocess import call
from urllib import parse
from http.client import HTTPSConnection
from bs4 import BeautifulSoup
from replbuilder import ReplCommand
from .votes import hn_get, get_pointer_parser


EDITOR = os.environ.get('EDITOR', 'vim')


def send_comment_request(form_data, user_cookie):
    headers = {"Cookie": user_cookie}
    data = parse.urlencode(form_data).encode()
    conn = HTTPSConnection("news.ycombinator.com")
    conn.request("POST", "/comment", data, headers=headers)
    resp = conn.getresponse()
    if str(resp.code).startswith("4"):
        print("Call REJECTED: {}".format(resp.read()))
        raise "Call is unsuccessful"
    conn.close()


def get_reply_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("pointer", type=int, help="Pointers from previous command")
    parser.add_argument("-s", "--comment", type=str, help="Your reply (comment) here as string, you can also choose to use editor")
    parser.add_argument("-e", "--editor", action="store_true", default=False, help="Use editor (e.g. vim) to edit the reply")
    return parser


def send_reply_form(item_id, text, context, is_story=True):
    if is_story:
        sub_link = "item?id={}".format(item_id)
    else:
        sub_link = "reply?id={}".format(item_id)
    reply_page = hn_get(sub_link, context.user_cookie)
    soup = BeautifulSoup(reply_page, "html.parser")
    comment_form = soup.select('form[action="comment"]')[0]
    hmac_element = comment_form.select('input[name="hmac"]')[0]
    hmac = hmac_element["value"]
    form_data = {
        "parent": item_id,
        "hmac": hmac,
        "text": text,
    }
    if is_story:
        form_data["goto"] = sub_link
    send_comment_request(form_data, context.user_cookie)
    print("\033[0;32mComment Published\033[0m")
    sys.stdout.flush()
    context.drop_item(item_id) # Parent will need to be reloaded


def editor_edit(item):
    if item["type"] == "comment":
        text = item["text"]
        text = html.unescape(text)
        text = text.replace("<p>", "\n")
    else:
        text = item["title"]
    with tempfile.NamedTemporaryFile(suffix=".tmp") as tf:
        tf.write("YOU ARE REPLYING TO:\n\n".encode())
        tf.write(text.encode())
        tf.write("\n\n=== draft your reply below this line, do not delete this or above ===\n".encode())
        tf.flush()
        call([EDITOR, '+set backupcopy=yes', tf.name])
        edited_message = tf.read()
    return edited_message


def reply_to(args, context):
    if context.user_cookie is None:
        raise ValueError("Cannot reply if not logged in")
    item_id = context.current_pointers[args.pointer]
    context.store_item(item_id)
    item = context.loaded_items[item_id]
    comment_string = ""
    if args.editor:
        comment_string = editor_edit(item)
        print("\033[1;32mEDITOR OUTPUT:\033[0m")
        print(comment_string)
    elif args.comment:
        comment_string = args.comment
    else:
        raise ValueError("Reply must contain content")
    if item["type"] == "comment":
        send_reply_form(item_id, comment_string, context, False)
    else:
        send_reply_form(item_id, comment_string, context)


reply_to_command = ReplCommand("reply_to", get_reply_parser(), reply_to, "Reply to a story or a comment, can use editor of choice", use_context=True)



from .get_comments import get_tree_command
from .get_front_page import get_front_page_command
from .get_story import get_story_command
from .get_user import get_user_command
from .login import login_command, logout_command
from .votes import upvote_command, downvote_command, unvote_command
from .reply import reply_to_command

READ_COMMANDS = [
    get_front_page_command,
    get_tree_command,
    get_story_command,
    get_user_command
]

WRITE_COMMANDS = [
    login_command,
    logout_command,
    upvote_command,
    downvote_command,
    unvote_command,
    reply_to_command
]



import argparse
from http.client import HTTPSConnection
from urllib import parse
from replbuilder import ReplCommand


def login_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("username", type=str, help="Your user name")
    parser.add_argument("password", type=str, help="Your password")
    return parser


def login(args, context):
    if context.user_cookie is not None:
        curr_user = context.user_cookie.split("=")[1].split("&")[0]
        if curr_user == args.username:
            print("You are already logged in, nothing to do")
            return
    login_data = parse.urlencode({
        "acct": args.username,
        "pw": args.password
    }).encode()
    try:
        conn = HTTPSConnection("news.ycombinator.com")
        conn.request("POST", "/login", login_data)
        resp = conn.getresponse()
        login_cookie = resp.info().get_all("Set-Cookie")[0]
        user_cookie = login_cookie.split("; ")[0]
        context.store_user_cookie(user_cookie)
        print("Successfully logged in as {}".format(args.username))
    except Exception as e:
        print("Login failed: {}".format(e))
    finally:
        conn.close()


def logout(args, context):
    if context.user_cookie is not None:
        curr_user = context.user_cookie.split("=")[1].split("&")[0]
        print("Logging out as {}".format(curr_user))
        context.store_user_cookie(None)
    else:
        print("Not logged in, nothing to do")


login_command = ReplCommand("login", login_parser(), login, "Login with username and password, this requests a user cookie which is destroyed when the cli exits", use_context=True)
logout_command = ReplCommand("logout", argparse.ArgumentParser(), logout, "Logout, wipes the user cookie", use_context=True)
